@@ -9,17 +9,28 @@ For example, you can check who you are logged in as with
 
 >>> username = beaker.account.whoami().name
 
+.. important::
+    In this example - and all other examples - ``beaker`` is an instance of the
+    :class:`Beaker` client class, not the :mod:`beaker` module.
+    See `Quick start <./quickstart.html>`_ to learn how to instantiate the client.
+
 Organizations
 -------------
 
 Manage Beaker organizations with :data:`Beaker.organization`.
 
 For example, you can get information about an organization with
-:meth:`Beaker.organization.get() <services.OrganizationClient>`:
+:meth:`Beaker.organization.get() <services.OrganizationClient.get>`:
 
 >>> beaker.organization.get(beaker_org_name).display_name
 'AI2'
 
+You can also add, get, list, or remove members with
+:meth:`Beaker.organization.add_member() <services.OrganizationClient.add_member>`,
+:meth:`.get_member() <services.OrganizationClient.get_member>`,
+:meth:`.list_members() <services.OrganizationClient.list_members>`, or
+:meth:`.remove_member() <services.OrganizationClient.remove_member>`, respectively.
+
 Workspaces
 ----------
 
@@ -27,13 +38,24 @@ Manage Beaker workspaces with :data:`Beaker.workspace`.
 
 For example, you can create a workspace with :meth:`Beaker.workspace.ensure() <services.WorkspaceClient.ensure>`:
 
->>> beaker.workspace.ensure(workspace_name)
+>>> workspace = beaker.workspace.ensure(workspace_name)
 
-And you can retreive metadata about a workspace with :meth:`Beaker.workspace.get() <services.WorkspaceClient.get>`:
+You can retreive metadata about a workspace with :meth:`Beaker.workspace.get() <services.WorkspaceClient.get>`:
 
 >>> beaker.workspace.get(workspace_name).id
 '01FPB5S64Y649S1948QHQHVCVE'
 
+You can list datasets in a workspace with
+:meth:`Beaker.workspace.datasets() <services.WorkspaceClient.datasets>`:
+
+>>> datasets = beaker.workspace.datasets(workspace_name, results=False)
+
+Similarly, you can list experiments or images with
+:meth:`Beaker.workspace.experiments() <services.WorkspaceClient.experiments>`
+or
+:meth:`Beaker.workspace.images() <services.WorkspaceClient.images>`,
+respectively.
+
 Clusters
 --------
 
@@ -45,7 +67,8 @@ For example, you can get information about a cluster with
 >>> beaker.cluster.get(beaker_cluster_name).autoscale
 True
 
-Or you could check how many GPUs are free on an on-premise cluster with:
+Or you could check how many GPUs are free on an on-premise cluster with
+:meth:`Beaker.cluster.utilization() <services.ClusterClient.utilization>`:
 
 >>> free_gpus = 0
 >>> for node_util in beaker.cluster.utilization(beaker_on_prem_cluster_name):
@@ -54,7 +77,7 @@ Or you could check how many GPUs are free on an on-premise cluster with:
 Nodes
 -----
 
-Manage Beaker nodes.
+Manage Beaker nodes with :data:`Beaker.node`.
 
 For example, you can get information about a node with
 :meth:`Beaker.node.get() <services.NodeClient.get>`:
@@ -106,23 +129,29 @@ For example, create an experiment with :meth:`Beaker.experiment.create() <servic
 ...     workspace=workspace_name,
 ... )
 
-Wait for the experiment to complete:
+Wait for the experiment to complete with
+:meth:`Beaker.experiment.await_all() <services.ExperimentClient.await_all>`:
 
 >>> experiment = beaker.experiment.await_all(
-...     experiment.full_name,
+...     experiment,
 ...     timeout=60 * 3,
 ...     quiet=True,
-... )
+... )[0]
 <BLANKLINE>
 
 Get the logs from the experiment with :meth:`Beaker.experiment.logs() <services.ExperimentClient.logs>`:
 
 >>> logs = "".join([
 ...    line.decode() for line in
-...    beaker.experiment.logs(experiment.full_name, quiet=True)
+...    beaker.experiment.logs(experiment, quiet=True)
 ... ])
 <BLANKLINE>
 
+Get the results from all tasks in an experiment with
+:meth:`Beaker.experiment.results <services.ExperimentClient.results>`:
+
+>>> task, results = beaker.experiment.results(experiment)[0]
+
 Datasets
 --------
 
@@ -147,6 +176,16 @@ And download a dataset with :meth:`Beaker.dataset.fetch() <services.DatasetClien
 >>> beaker.dataset.fetch(dataset, target=download_path, quiet=True)
 <BLANKLINE>
 
+Secrets
+-------
+
+Manage Beaker secrets with :data:`Beaker.secret`.
+
+For example, you can read, write, or delete secrets with
+:meth:`Beaker.secret.read() <services.SecretClient.read>`,
+:meth:`Beaker.secret.write() <services.SecretClient.write>`, and
+:meth:`Beaker.secret.delete() <services.SecretClient.delete>`, respectively.
+
 """
 
 from .client import *
