@@ -6,6 +6,7 @@ from typing import Any, Dict, List, Optional, Type, TypeVar
 from pydantic import BaseModel as _BaseModel
 from pydantic import Field, ValidationError, root_validator, validator
 
+from .aliases import PathOrStr
 from .util import to_lower_camel
 
 T = TypeVar("T")
@@ -14,6 +15,7 @@ logger = logging.getLogger(__name__)
 
 
 SPEC_VERSION = "v2-alpha"
+ALLOWED_SPEC_VERSIONS = {SPEC_VERSION, "v2"}
 
 
 class BaseModel(_BaseModel):
@@ -307,6 +309,8 @@ class TaskSpec(BaseModel):
     A Beaker experiment may contain multiple tasks.
     A task may also depend on the results of another task in its experiment,
     creating an execution graph.
+
+    A :class:`TaskSpec` defines a :class:`Task`.
     """
 
     image: ImageSource
@@ -368,7 +372,9 @@ class TaskSpec(BaseModel):
 
 class ExperimentSpec(BaseModel):
     """
-    An experiment is the main unit of execution in Beaker.
+    Experiments are the main unit of execution in Beaker.
+
+    An :class:`ExperimentSpec` defines an :class:`Experiment`.
     """
 
     tasks: List[TaskSpec]
@@ -388,10 +394,21 @@ class ExperimentSpec(BaseModel):
 
     @validator("version")
     def _validate_version(cls, v: str) -> str:
-        if v != SPEC_VERSION:
-            raise ValueError(f"Only version '{SPEC_VERSION}' is currently supported")
+        if v not in ALLOWED_SPEC_VERSIONS:
+            raise ValueError(f"Spec version '{v}' is not supported")
         return v
 
+    @classmethod
+    def from_file(cls, path: PathOrStr) -> "ExperimentSpec":
+        """
+        Load an :class:`ExperimentSpec` from a YAML file.
+        """
+        import yaml
+
+        with open(path) as spec_file:
+            raw_spec = yaml.load(spec_file, Loader=yaml.SafeLoader)
+            return cls.from_json(raw_spec)
+
 
 class WorkspaceSize(BaseModel):
     datasets: int
@@ -578,6 +595,10 @@ class JobKind(BaseEnum):
 
 
 class Job(BaseModel):
+    """
+    A :class:`Job` is an execution of a :class:`Task`.
+    """
+
     id: str
     kind: JobKind
     author: Account
@@ -607,6 +628,17 @@ class Experiment(BaseModel):
     jobs: List[Job] = Field(default_factory=list)
 
 
+class Task(BaseModel):
+    id: str
+    experiment_id: str
+    owner: Account
+    author: Account
+    created: datetime
+    name: Optional[str] = None
+    schedulable: bool = False
+    jobs: List[Job] = Field(default_factory=list)
+
+
 class DatasetStorage(BaseModel):
     id: str
     address: str
@@ -654,10 +686,18 @@ class DatasetStorageInfo(BaseModel):
 
 class FileInfo(BaseModel):
     path: str
-    size: int
     digest: str
     updated: datetime
-    url: str
+
+    size: Optional[int] = None
+    """
+    The size of the file, if known.
+    """
+
+    url: Optional[str] = None
+    """
+    A URL that can be used to directly download the file.
+    """
 
 
 class DatasetManifest(BaseModel):
@@ -681,3 +721,29 @@ class Image(BaseModel):
         if v is not None and v.year == 1:
             return None
         return v
+
+
+class Secret(BaseModel):
+    name: str
+    created: datetime
+    updated: datetime
+
+
+class ImagesPage(BaseModel):
+    data: List[Image]
+    next_cursor: Optional[str] = None
+
+
+class DatasetsPage(BaseModel):
+    data: List[Dataset]
+    next_cursor: Optional[str] = None
+
+
+class ExperimentsPage(BaseModel):
+    data: List[Experiment]
+    next_cursor: Optional[str] = None
+
+
+class WorkspacePage(BaseModel):
+    data: List[Workspace]
+    next_cursor: Optional[str] = None
