@@ -20,6 +20,10 @@ class OrganizationNotFound(BeakerError):
     pass
 
 
+class OrganizationNotSet(BeakerError):
+    pass
+
+
 class ConfigurationError(BeakerError):
     pass
 
@@ -82,3 +86,11 @@ class WorkspaceNotSet(BeakerError):
 
 class NodeNotFound(BeakerError):
     pass
+
+
+class DatasetWriteError(BeakerError):
+    pass
+
+
+class SecretNotFound(BeakerError):
+    pass
