@@ -6,6 +6,10 @@ from .service_client import ServiceClient
 
 
 class ClusterClient(ServiceClient):
+    """
+    Accessed via :data:`Beaker.cluster <beaker.Beaker.cluster>`.
+    """
+
     def create(
         self,
         name: str,
@@ -115,22 +119,25 @@ class ClusterClient(ServiceClient):
             exceptions_for_status={404: ClusterNotFound(self._not_found_err_msg(cluster_id))},
         )
 
-    def list(self, org: Union[str, Organization]) -> List[Cluster]:
+    def list(self, org: Optional[Union[str, Organization]] = None) -> List[Cluster]:
         """
         List clusters under an organization.
 
-        :param org: The organization name or object.
+        :param org: The organization name or object. If not specified,
+            :data:`Beaker.config.default_org <beaker.Config.default_org>` is used.
 
         :raises OrganizationNotFound: If the organization doesn't exist.
+        :raises OrganizationNotSet: If neither ``org`` nor
+            :data:`Beaker.config.default_org <beaker.Config.default_org>` are set.
         :raises HTTPError: Any other HTTP exception that can occur.
         """
-        org_name = org if isinstance(org, str) else org.name
+        org: Organization = self._resolve_org(org)
         return [
             Cluster.from_json(d)
             for d in self.request(
-                f"clusters/{self._url_quote(org_name)}",
+                f"clusters/{org.id}",
                 method="GET",
-                exceptions_for_status={404: OrganizationNotFound(org_name)},
+                exceptions_for_status={404: OrganizationNotFound(org.id)},
             ).json()["data"]
         ]
 
@@ -197,6 +204,48 @@ class ClusterClient(ServiceClient):
             )
         return out
 
+    def filter_available(
+        self, resources: TaskResources, *clusters: Union[str, Cluster]
+    ) -> List[Cluster]:
+        """
+        Filter clusters with enough free resources to run a job with the requested resources.
+
+        :param resources: The requested task resources.
+        :param clusters: Clusters to inspect and filter.
+
+        :raises ClusterNotFound: If one of the clusters doesn't exist.
+        """
+
+        def is_compat(node_spec: Union[NodeSpec, NodeShape, NodeSpecUtil]) -> bool:
+            if resources.gpu_count and (
+                node_spec.gpu_count is None or node_spec.gpu_count < resources.gpu_count
+            ):
+                return False
+            if resources.cpu_count and (
+                node_spec.cpu_count is None or node_spec.cpu_count < resources.cpu_count
+            ):
+                return False
+            # TODO: check memory too
+            return True
+
+        available: List[Cluster] = []
+        for cluster_ in clusters:
+            cluster: Cluster = cluster_ if isinstance(cluster_, Cluster) else self.get(cluster_)
+
+            if cluster.node_shape is not None and not is_compat(cluster.node_shape):
+                continue
+
+            node_utilization = self.utilization(cluster)
+            if cluster.autoscale and len(node_utilization) < cluster.capacity:
+                available.append(cluster)
+            else:
+                for node_util in node_utilization:
+                    if is_compat(node_util.free):
+                        available.append(cluster)
+                        break
+
+        return available
+
     def _not_found_err_msg(self, cluster: str) -> str:
         return (
             f"'{cluster}': Make sure you're using the *full* name of the cluster "
