@@ -1,6 +1,7 @@
 import os
+from datetime import datetime
 from pathlib import Path
-from typing import Deque, Dict, Generator, List, Optional, Union
+from typing import Deque, Dict, Generator, List, Optional, Tuple, Union
 
 from rich.progress import (
     BarColumn,
@@ -20,10 +21,15 @@ from .service_client import ServiceClient
 
 
 class DatasetClient(ServiceClient):
+    """
+    Accessed via :data:`Beaker.dataset <beaker.Beaker.dataset>`.
+    """
+
     HEADER_UPLOAD_ID = "Upload-ID"
     HEADER_UPLOAD_LENGTH = "Upload-Length"
     HEADER_UPLOAD_OFFSET = "Upload-Offset"
     HEADER_DIGEST = "Digest"
+    HEADER_LAST_MODIFIED = "Last-Modified"
 
     SHA256 = "SHA256"
 
@@ -32,7 +38,7 @@ class DatasetClient(ServiceClient):
     def create(
         self,
         name: str,
-        source: PathOrStr,
+        *sources: PathOrStr,
         target: Optional[PathOrStr] = None,
         workspace: Optional[str] = None,
         force: bool = False,
@@ -44,10 +50,9 @@ class DatasetClient(ServiceClient):
         Create a dataset with the source file(s).
 
         :param name: The name to assign to the new dataset.
-        :param source: The local source file or directory of the dataset.
-        :param target: If ``source`` is a file, you can change the name of the file in the dataset
-            by specifying ``target``. If ``source`` is a directory, you can set ``target`` to
-            specify a directory in the dataset to upload ``source`` under.
+        :param sources: Local source files or directories to upload to the dataset.
+        :param target: If specified, all source files/directories will be uploaded under
+            a directory of this name.
         :param workspace: The workspace to upload the dataset to. If not specified,
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` is used.
         :param force: If ``True`` and a dataset by the given name already exists, it will be overwritten.
@@ -60,16 +65,11 @@ class DatasetClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.defeault_workspace <beaker.Config.default_workspace>` are set.
         :raises HTTPError: Any other HTTP exception that can occur.
-        :raises UnexpectedEOFError: If ``source`` is a single empty file, or if ``source`` is a directory and
-            the contents of one of the directories files is changed while creating the dataset.
-        :raises FileNotFoundError: If the ``source`` doesn't exist.
+        :raises UnexpectedEOFError: If a source file is an empty file, or if a source is a directory and
+            the contents of one of the directory's files changes while creating the dataset.
+        :raises FileNotFoundError: If a source doesn't exist.
         """
-        workspace_name = self._resolve_workspace(workspace)
-
-        # Ensure source exists.
-        source: Path = Path(source)
-        if not source.exists():
-            raise FileNotFoundError(str(source))
+        workspace: Workspace = self._resolve_workspace(workspace)
 
         # Create the dataset.
         def make_dataset() -> Dataset:
@@ -78,7 +78,10 @@ class DatasetClient(ServiceClient):
                     "datasets",
                     method="POST",
                     query={"name": name},
-                    data={"workspace": workspace_name, "fileheap": True},
+                    data={
+                        "workspace": workspace.id,  # type: ignore
+                        "fileheap": True,
+                    },
                     exceptions_for_status={409: DatasetConflict(name)},
                 ).json()
             )
@@ -94,7 +97,7 @@ class DatasetClient(ServiceClient):
         assert dataset_info.storage is not None
 
         # Upload the file(s).
-        self._sync_source(dataset_info, source, target, quiet=quiet, max_workers=max_workers)
+        self.sync(dataset_info, *sources, target=target, quiet=quiet, max_workers=max_workers)
 
         # Commit the dataset.
         if commit:
@@ -103,7 +106,7 @@ class DatasetClient(ServiceClient):
         # Return info about the dataset.
         return self.get(dataset_info.id)
 
-    def commit(self, dataset: Union[str, Dataset]):
+    def commit(self, dataset: Union[str, Dataset]) -> Dataset:
         """
         Commit the dataset.
 
@@ -113,10 +116,12 @@ class DatasetClient(ServiceClient):
         :raises HTTPError: Any other HTTP exception that can occur.
         """
         dataset_id = dataset if isinstance(dataset, str) else dataset.id
-        self.request(
-            f"datasets/{self._url_quote(dataset_id)}",
-            method="PATCH",
-            data={"commit": True},
+        return Dataset.from_json(
+            self.request(
+                f"datasets/{self._url_quote(dataset_id)}",
+                method="PATCH",
+                data={"commit": True},
+            ).json()
         )
 
     def fetch(
@@ -187,13 +192,19 @@ class DatasetClient(ServiceClient):
             with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
                 download_futures = []
                 for file_info in self._iter_files(dataset.storage):
+                    assert file_info.size is not None
                     if total_bytes_to_download is None:
                         progress.update(bytes_task, total=total_downloaded + file_info.size + 1)
                     target_path = target / Path(file_info.path)
                     if not force and target_path.exists():
                         raise FileExistsError(file_info.path)
                     future = executor.submit(
-                        self._download_file, file_info, target_path, progress, bytes_task
+                        self._download_file,
+                        dataset.storage,
+                        file_info,
+                        target_path,
+                        progress,
+                        bytes_task,
                     )
                     download_futures.append(future)
 
@@ -203,6 +214,49 @@ class DatasetClient(ServiceClient):
             if total_bytes_to_download is None:
                 progress.update(bytes_task, total=total_downloaded, completed=total_downloaded)
 
+    def stream_file(
+        self,
+        dataset: Union[str, Dataset],
+        file_name: str,
+        offset: int = 0,
+        length: int = -1,
+        max_retries: int = 5,
+    ) -> Generator[bytes, None, None]:
+        """
+        Stream download the contents of a single file from a dataset.
+
+        :param dataset: The dataset ID, full name, or object.
+        :param file_name: The path of the file within the dataset.
+        :param offset: Offset to start from, in bytes.
+        :param length: Number of bytes to read.
+        :param max_retries: Number of times to restart the download when HTTP errors occur.
+            Errors can be expected for very large files.
+
+        :raises DatasetNotFound: If the dataset can't be found.
+        :raises FileNotFoundError: If the file doesn't exist in the dataset.
+        :raises HTTPError: Any other HTTP exception that can occur.
+        """
+        if not isinstance(dataset, Dataset) or dataset.storage is None:
+            dataset = self.get(dataset.id if isinstance(dataset, Dataset) else dataset)
+            assert dataset.storage is not None
+        response = self.request(
+            f"datasets/{dataset.storage.id}/files/{file_name}",
+            method="HEAD",
+            token=dataset.storage.token,
+            base_url=dataset.storage.address,
+            exceptions_for_status={404: FileNotFoundError(file_name)},
+        )
+        file_info = FileInfo(
+            path=file_name,
+            digest=response.headers[self.HEADER_DIGEST],
+            updated=datetime.strptime(
+                response.headers[self.HEADER_LAST_MODIFIED], "%a, %d %b %Y %H:%M:%S %Z"
+            ),
+        )
+        yield from self._stream_file(
+            dataset.storage, file_info, offset=offset, length=length, max_retries=max_retries
+        )
+
     def get(self, dataset: str) -> Dataset:
         """
         Get info about a dataset.
@@ -236,21 +290,34 @@ class DatasetClient(ServiceClient):
             exceptions_for_status={404: DatasetNotFound(self._not_found_err_msg(dataset_id))},
         )
 
-    def _not_found_err_msg(self, dataset: str) -> str:
-        return (
-            f"'{dataset}': Make sure you're using a valid Beaker dataset ID or the "
-            f"*full* name of the dataset (with the account prefix, e.g. 'username/dataset_name')"
-        )
-
-    def _sync_source(
+    def sync(
         self,
-        dataset: Dataset,
-        source: PathOrStr,
+        dataset: Union[str, Dataset],
+        *sources: PathOrStr,
         target: Optional[PathOrStr] = None,
         quiet: bool = False,
         max_workers: int = 8,
     ) -> None:
-        source: Path = Path(source)
+        """
+        Sync local files or directories to an uncommitted dataset.
+
+        :param dataset: The dataset ID, full name, or object.
+        :param sources: Local source files or directories to upload to the dataset.
+        :param target: If specified, all source files/directories will be uploaded under
+            a directory of this name.
+        :param max_workers: The maximum number of thread pool workers to use to upload files concurrently.
+        :param quiet: If ``True``, progress won't be displayed.
+
+        :raises DatasetNotFound: If the dataset can't be found.
+        :raises DatasetWriteError: If the dataset was already committed.
+        :raises FileNotFoundError: If a source doesn't exist.
+        :raises UnexpectedEOFError: If a source is an empty file, or if a source is a directory and
+            the contents of one of the directory's files changes while creating the dataset.
+        :raises HTTPError: Any other HTTP exception that can occur.
+        """
+        dataset: Dataset = self.get(dataset.id if isinstance(dataset, Dataset) else dataset)
+        if dataset.committed is not None:
+            raise DatasetWriteError(dataset.id)
 
         with Progress(
             "[progress.description]{task.description}",
@@ -262,62 +329,125 @@ class DatasetClient(ServiceClient):
             disable=quiet,
         ) as progress:
             bytes_task = progress.add_task("Uploading dataset")
-            if source.is_file():
-                size = source.lstat().st_size
-                if size == 0:
-                    raise UnexpectedEOFError(str(source))
-                progress.update(bytes_task, total=size)
-                total_uploaded = self._upload_file(
-                    dataset, size, source, target or source.name, progress, bytes_task
-                )
-                progress.update(bytes_task, total=total_uploaded)
-            elif source.is_dir():
-                import concurrent.futures
-
-                # Gather all files to upload and count the total number of bytes.
-                total_bytes = 0
-                path_to_size: Dict[Path, int] = {}
-                for path in source.glob("**/*"):
-                    if path.is_dir():
-                        continue
-                    size = path.lstat().st_size
+            total_bytes = 0
+            # map source path to (target_path, size)
+            path_info: Dict[Path, Tuple[Path, int]] = {}
+            for name in sources:
+                source = Path(name)
+                if source.is_file():
+                    target_path = Path(source.name)
+                    if target is not None:
+                        target_path = Path(target) / target_path
+                    size = source.lstat().st_size
                     if size == 0:
-                        continue
-                    path_to_size[path] = size
+                        raise UnexpectedEOFError(str(source))
+                    path_info[source] = (target_path, size)
                     total_bytes += size
-                progress.update(bytes_task, total=total_bytes)
-
-                # Now upload.
-                with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
-                    # Dispatch tasks to thread pool executor.
-                    future_to_path = {}
-                    for path, size in path_to_size.items():
+                elif source.is_dir():
+                    for path in source.glob("**/*"):
+                        if path.is_dir():
+                            continue
                         target_path = path.relative_to(source)
                         if target is not None:
                             target_path = Path(target) / target_path
-                        future = executor.submit(
-                            self._upload_file,
-                            dataset,
-                            size,
-                            path,
-                            target_path,
-                            progress,
-                            bytes_task,
-                            True,
-                        )
-                        future_to_path[future] = path
+                        size = path.lstat().st_size
+                        if size == 0:
+                            continue
+                        path_info[path] = (target_path, size)
+                        total_bytes += size
+                else:
+                    raise FileNotFoundError(source)
 
-                    # Collect completed tasks.
-                    for future in concurrent.futures.as_completed(future_to_path):
-                        path = future_to_path[future]
-                        original_size = path_to_size[path]
-                        actual_size = future.result()
-                        if actual_size != original_size:
-                            # If the size of the file has changed since we started, adjust total.
-                            total_bytes += actual_size - original_size
-                            progress.update(bytes_task, total=total_bytes)
-            else:
-                raise FileNotFoundError(source)
+            import concurrent.futures
+
+            progress.update(bytes_task, total=total_bytes)
+
+            # Now upload.
+            with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
+                # Dispatch tasks to thread pool executor.
+                future_to_path = {}
+                for path, (target_path, size) in path_info.items():
+                    future = executor.submit(
+                        self._upload_file,
+                        dataset,
+                        size,
+                        path,
+                        target_path,
+                        progress,
+                        bytes_task,
+                        True,
+                    )
+                    future_to_path[future] = path
+
+                # Collect completed tasks.
+                for future in concurrent.futures.as_completed(future_to_path):
+                    path = future_to_path[future]
+                    original_size = path_info[path][1]
+                    actual_size = future.result()
+                    if actual_size != original_size:
+                        # If the size of the file has changed since we started, adjust total.
+                        total_bytes += actual_size - original_size
+                        progress.update(bytes_task, total=total_bytes)
+
+    def ls(self, dataset: Union[str, Dataset]) -> Generator[FileInfo, None, None]:
+        """
+        List files in a dataset.
+
+        :param dataset: The dataset ID, full name, or object.
+
+        :raises DatasetNotFound: If the dataset can't be found.
+        :raises HTTPError: Any other HTTP exception that can occur.
+        """
+        if not isinstance(dataset, Dataset) or dataset.storage is None:
+            dataset = self.get(dataset.id if isinstance(dataset, Dataset) else dataset)
+            assert dataset.storage is not None
+        for file_info in self._iter_files(dataset.storage):
+            yield file_info
+
+    def size(self, dataset: Union[str, Dataset]) -> int:
+        """
+        Calculate the size of a dataset, in bytes.
+
+        :param dataset: The dataset ID, full name, or object.
+
+        :raises DatasetNotFound: If the dataset can't be found.
+        :raises HTTPError: Any other HTTP exception that can occur.
+        """
+        total = 0
+        for file_info in self.ls(dataset):
+            assert file_info.size is not None
+            total += file_info.size
+        return total
+
+    def rename(self, dataset: Union[str, Dataset], name: str) -> Dataset:
+        """
+        Rename a dataset.
+
+        :param dataset: The dataset ID, full name, or object.
+        :param name: The new name of the dataset.
+
+        :raises DatasetNotFound: If the dataset can't be found.
+        :raises DatasetConflict: If a dataset by that name already exists.
+        :raises HTTPError: Any other HTTP exception that can occur.
+        """
+        dataset_id = dataset if isinstance(dataset, str) else dataset.id
+        return Dataset.from_json(
+            self.request(
+                f"datasets/{self._url_quote(dataset_id)}",
+                method="PATCH",
+                data={"name": name},
+                exceptions_for_status={
+                    409: DatasetConflict(name),
+                    404: DatasetNotFound(dataset_id),
+                },
+            ).json()
+        )
+
+    def _not_found_err_msg(self, dataset: str) -> str:
+        return (
+            f"'{dataset}': Make sure you're using a valid Beaker dataset ID or the "
+            f"*full* name of the dataset (with the account prefix, e.g. 'username/dataset_name')"
+        )
 
     def _upload_file(
         self,
@@ -423,28 +553,67 @@ class DatasetClient(ServiceClient):
                 if not cursor:
                     last_request = True
 
+    def _stream_file(
+        self,
+        storage: DatasetStorage,
+        file_info: FileInfo,
+        chunk_size: int = 1,
+        offset: int = 0,
+        length: int = -1,
+        max_retries: int = 5,
+    ) -> Generator[bytes, None, None]:
+        def stream_file(offset: int, length: int) -> Generator[bytes, None, None]:
+            headers = {}
+            if offset > 0 and length > 0:
+                headers["Range"] = f"bytes={offset}-{offset + length - 1}"
+            elif offset > 0:
+                headers["Range"] = f"bytes={offset}-"
+            response = self.request(
+                f"datasets/{storage.id}/files/{file_info.path}",
+                method="GET",
+                stream=True,
+                headers=headers,
+                token=storage.token,
+                base_url=storage.address,
+                exceptions_for_status={404: FileNotFoundError(file_info.path)},
+            )
+            for chunk in response.iter_content(chunk_size=chunk_size):
+                yield chunk
+
+        retries = 0
+        while True:
+            try:
+                for chunk in stream_file(offset, length):
+                    offset += len(chunk)
+                    yield chunk
+                break
+            except HTTPError:
+                if retries >= max_retries:
+                    raise
+                retries += 1
+
     def _download_file(
-        self, file_info: FileInfo, target_path: Path, progress: Progress, task_id: TaskID
+        self,
+        storage: DatasetStorage,
+        file_info: FileInfo,
+        target_path: Path,
+        progress: Progress,
+        task_id: TaskID,
     ) -> int:
         import tempfile
 
         total_bytes = 0
         target_dir = target_path.parent
         target_dir.mkdir(exist_ok=True, parents=True)
-        with self._session_with_backoff() as session:
-            response = session.get(file_info.url, stream=True)
-            response.raise_for_status()
-            tmp_target = tempfile.NamedTemporaryFile(
-                "w+b", dir=target_dir, delete=False, suffix=".tmp"
-            )
-            try:
-                for chunk in response.iter_content(chunk_size=1024):
-                    total_bytes += len(chunk)
-                    tmp_target.write(chunk)
-                    progress.update(task_id, advance=len(chunk))
-                os.replace(tmp_target.name, target_path)
-            finally:
-                tmp_target.close()
-                if os.path.exists(tmp_target.name):
-                    os.remove(tmp_target.name)
+        tmp_target = tempfile.NamedTemporaryFile("w+b", dir=target_dir, delete=False, suffix=".tmp")
+        try:
+            for chunk in self._stream_file(storage, file_info):
+                total_bytes += len(chunk)
+                tmp_target.write(chunk)
+                progress.update(task_id, advance=len(chunk))
+            os.replace(tmp_target.name, target_path)
+        finally:
+            tmp_target.close()
+            if os.path.exists(tmp_target.name):
+                os.remove(tmp_target.name)
         return total_bytes
